@@ -27,7 +27,7 @@
  * Known FOURCCs: 'IV41'
  */
 
-#define ALT_BITSTREAM_READER_LE
+#define BITSTREAM_READER_LE
 #include "avcodec.h"
 #include "get_bits.h"
 #include "dsputil.h"
@@ -94,7 +94,7 @@ typedef struct {
 } IVI4DecContext;
 
 
-struct {
+static const struct {
     InvTransformPtr *inv_trans;
     DCTransformPtr  *dc_trans;
     int             is_2d_trans;
@@ -148,7 +148,7 @@ static int decode_plane_subdivision(GetBitContext *gb)
 
 static inline int scale_tile_size(int def_size, int size_factor)
 {
-    return (size_factor == 15 ? def_size : (size_factor + 1) << 5);
+    return size_factor == 15 ? def_size : (size_factor + 1) << 5;
 }
 
 /**
@@ -160,7 +160,7 @@ static inline int scale_tile_size(int def_size, int size_factor)
  */
 static int decode_pic_hdr(IVI4DecContext *ctx, AVCodecContext *avctx)
 {
-    int             pic_size_indx, val, i, p;
+    int             pic_size_indx, i, p;
     IVIPicConfig    pic_conf;
 
     if (get_bits(&ctx->gb, 18) != 0x3FFF8) {
@@ -301,7 +301,7 @@ static int decode_pic_hdr(IVI4DecContext *ctx, AVCodecContext *avctx)
     /* skip picture header extension if any */
     while (get_bits1(&ctx->gb)) {
         av_dlog(avctx, "Pic hdr extension encountered!\n");
-        val = get_bits(&ctx->gb, 8);
+        skip_bits(&ctx->gb, 8);
     }
 
     if (get_bits1(&ctx->gb)) {
@@ -325,7 +325,7 @@ static int decode_pic_hdr(IVI4DecContext *ctx, AVCodecContext *avctx)
 static int decode_band_hdr(IVI4DecContext *ctx, IVIBandDesc *band,
                            AVCodecContext *avctx)
 {
-    int plane, band_num, hdr_size, indx, transform_id, scan_indx;
+    int plane, band_num, indx, transform_id, scan_indx;
     int i;
 
     plane    = get_bits(&ctx->gb, 2);
@@ -337,7 +337,10 @@ static int decode_band_hdr(IVI4DecContext *ctx, IVIBandDesc *band,
 
     band->is_empty = get_bits1(&ctx->gb);
     if (!band->is_empty) {
-        hdr_size = get_bits1(&ctx->gb) ? get_bits(&ctx->gb, 16) : 4;
+        /* skip header size
+         * If header size is not given, header size is 4 bytes. */
+        if (get_bits1(&ctx->gb))
+            skip_bits(&ctx->gb, 16);
 
         band->is_halfpel = get_bits(&ctx->gb, 2);
         if (band->is_halfpel >= 2) {
@@ -369,7 +372,8 @@ static int decode_band_hdr(IVI4DecContext *ctx, IVIBandDesc *band,
 
         if (!get_bits1(&ctx->gb) || ctx->frame_type == FRAMETYPE_INTRA) {
             transform_id = get_bits(&ctx->gb, 5);
-            if (!transforms[transform_id].inv_trans) {
+            if (transform_id >= FF_ARRAY_ELEMS(transforms) ||
+                !transforms[transform_id].inv_trans) {
                 av_log_ask_for_sample(avctx, "Unimplemented transform: %d!\n", transform_id);
                 return AVERROR_PATCHWELCOME;
             }
